<?php

/**
 * @file
 * Common functions for the Subnodes by taxonomy module.
 */

/**
 * Obtain relation information.
 *
 * @param $parent_type
 *   The type of the parent node in the relationship.
 * @param $parent_nid
 *   Optional. The nid of a parent node.
 * @param $new_tid
 *   Optional. The term id for a new relation.
 *
 * @return
 *   An object with the requested relation information.
 *   FALSE if relation does not exist, or otherwise unable to obtain requested data.
 */
function _subnodes_by_taxonomy_get_relation($parent_type, $parent_nid = NULL, $new_tid = NULL) {
  // Obtain relation settings.
  $relation = db_fetch_object(db_query("SELECT parent_type, child_type, group_name, settings FROM {subnodes_by_taxonomy_relations} WHERE parent_type = '%s'", $parent_type));
  if (!isset($relation) || empty($relation->parent_type)) {
    return FALSE;
  }
  $relation->settings = unserialize($relation->settings);

  // Obtain types information.
  $relation->types = array(
    'parent' => node_get_types('type', $relation->parent_type),
    'child' => node_get_types('type', $relation->child_type),
  );
  _subnodes_by_taxonomy_localize_content_type($relation->types['parent']);
  _subnodes_by_taxonomy_localize_content_type($relation->types['child']);

  // Return basic relation settings if we don't need more information.
  if (isset($parent_nid) && $parent_nid === FALSE) {
    return $relation;
  }

  // Obtain CCK/Fieldgroup information.
  if (!_subnodes_by_taxonomy_load_content_data($relation, $parent_type)) {
    return FALSE;
  }

  // Obtain taxonomy information.
  if (!_subnodes_by_taxonomy_load_taxonomy_data($relation)) {
    return FALSE;
  }

  // If a parent nid has been given, then get information about children.
  if (isset($parent_nid)) {
    $nodereference_column = $relation->fields['nodereference']['column'];
    $nodereference_table = $relation->fields['nodereference']['table'];
    if ($nodereference_table == $relation->fields['content_taxonomy']['table']) {
      $content_taxonomy_column = 'nr.'. $relation->fields['content_taxonomy']['column'];
      $content_taxonomy_join = '';
    }
    else {
      $content_taxonomy_column = 'ct.'. $relation->fields['content_taxonomy']['column'];
      $content_taxonomy_join = 'INNER JOIN {'. $relation->fields['content_taxonomy']['table'] .'} ct ON ct.vid = n.vid AND ct.nid = n.nid';
    }
    $sql = 'SELECT n.vid, n.nid, '. $content_taxonomy_column .' AS tid
      FROM {node} n
        INNER JOIN {'. $nodereference_table .'} nr ON nr.vid = n.vid AND nr.nid = n.nid
        '. $content_taxonomy_join .'
      WHERE nr.'. $nodereference_column .' = %d';
    $result = db_query($sql, $parent_nid);
    while ($row = db_fetch_object($result)) {
      $row->name = $relation->terms[$row->tid]->name;
      $row->description = !empty($relation->terms[$row->tid]->description) ? $relation->terms[$row->tid]->description : $relation->terms[$row->tid]->name;
      $relation->tabset[$row->nid] = $row;
    }
    uasort($relation->tabset, '_subnodes_by_taxonomy_sort_tabs');

    // Add a new entry for the relation that's about to be created.
    if (isset($new_tid) && isset($relation->terms[$new_tid])) {
      $relation->tabset[0] = (object)array(
        'vid' => 0,
        'nid' => 0,
        'tid' => $new_tid,
        'name' => $relation->terms[$new_tid]->name,
      );
    }
  }

  return $relation;
}

/**
 * Obtain relation information for the given child.
 */
function subnodes_by_taxonomy_get_relation_from_child($child_type, $child_nid = NULL) {
  static $cache;
  if (!isset($cache)) {
    $cache = array();
  }

  // Build unique key for this request.
  $key = $child_type .'|'. $child_nid;

  if (!isset($cache[$key])) {
    $cache[$key] = FALSE;

    // Obtain relation settings.
    $relation = db_fetch_object(db_query("SELECT parent_type, child_type, group_name, settings FROM {subnodes_by_taxonomy_relations} WHERE child_type = '%s'", $child_type));
    if (!isset($relation) || empty($relation->parent_type)) {
      return FALSE;
    }
    $relation->settings = unserialize($relation->settings);

    // Obtain CCK/Fieldgroup information.
    if (!_subnodes_by_taxonomy_load_content_data($relation, $relation->parent_type)) {
      return FALSE;
    }

    if (isset($child_nid)) {
      $nodereference_column = $relation->fields['nodereference']['column'];
      $nodereference_table = $relation->fields['nodereference']['table'];
      if ($nodereference_table == $relation->fields['content_taxonomy']['table']) {
        $content_taxonomy_column = 'nr.'. $relation->fields['content_taxonomy']['column'];
        $content_taxonomy_join = '';
      }
      else {
        $content_taxonomy_column = 'ct.'. $relation->fields['content_taxonomy']['column'];
        $content_taxonomy_join = 'INNER JOIN {'. $relation->fields['content_taxonomy']['table'] .'} ct ON ct.vid = n.vid AND ct.nid = n.nid';
      }
      $sql = 'SELECT nr.'. $nodereference_column .' AS nid, '. $content_taxonomy_column .' AS tid
        FROM {node} n
          INNER JOIN {'. $nodereference_table .'} nr ON nr.vid = n.vid AND nr.nid = n.nid
          '. $content_taxonomy_join .'
        WHERE n.nid = %d';
      if (($result = db_query($sql, $child_nid)) && ($row = db_fetch_object($result))) {
        $relation->parent_data = array(
          'nid' => (int)$row->nid,
          'tid' => (int)$row->tid,
        );
      }
    }

    $cache[$key] = $relation;
  }
  return $cache[$key];
}

/**
 * Load relation information related to CCK/Fieldgroup.
 */
function _subnodes_by_taxonomy_load_content_data(&$relation, $parent_type) {
  // Obtain group information.
  $groups = fieldgroup_groups($relation->child_type);
  if (empty($groups) || empty($groups[$relation->group_name])) {
    return FALSE;
  }
  $group = $groups[$relation->group_name];

  // Save group related data.
  $relation->group = array(
    'name' => $relation->group_name,
    'label' => $group['label'],
  );

  // Obtain CCK information.
  $content_type = content_types($relation->child_type);
  if (empty($content_type['fields'])) {
    return FALSE;
  }

  // Parse only the fields in the relation group.
  $group_fields = array_intersect_key($content_type['fields'], $group['fields']);
  if (empty($group_fields)) {
    return FALSE;
  }

  // Search for a node reference and a taxonomy field in this group.
  $fields_data = array();
  foreach ($group_fields as $field_name => $field) {
    // Ignore inactive fields, locked fields and fields with multiple values.
    if (!$field['active'] || $field['locked'] || $field['multiple']) {
      continue;
    }
    $field_type = $field['type'];
    if ($field_type == 'content_taxonomy') {
      if (empty($field['vid'])) {
        continue;
      }
    }
    elseif ($field_type == 'nodereference') {
      if (!isset($field['referenceable_types']) || !$field['referenceable_types'][$parent_type]) {
        continue;
      }
    }
    else {
      continue;
    }
    $db_info = content_database_info($field);
    $fields_data[$field_type] = array(
      'name' => $field_name,
      'type' => $field_type,
      'label' => $field['widget']['label'],
      'table' => $db_info['table'],
      'column' => $field_name .'_'. current(array_keys($db_info['columns'])),
    );
    if ($field_type == 'content_taxonomy') {
      $fields_data[$field_type]['vid'] = $field['vid'];
    }
  }

  // Do we have the required number of fields?
  if (count($fields_data) != 2) {
    return FALSE;
  }

  // Store collected data into relation structure.
  $relation->fields = $fields_data;

  return TRUE;
}

/**
 * Load relation information related to CCK/Fieldgroup.
 */
function _subnodes_by_taxonomy_load_taxonomy_data(&$relation) {
  // Do we have a vocabulary and terms?
  if (!($vocabulary = taxonomy_vocabulary_load($relation->fields['content_taxonomy']['vid']))) {
    return FALSE;
  }
  if (!($terms = taxonomy_get_children(0, $relation->fields['content_taxonomy']['vid']))) {
    return FALSE;
  }

  if (module_exists('i18ntaxonomy') && i18ntaxonomy_vocabulary($vocabulary->vid) == I18N_TAXONOMY_LOCALIZE) {
    $vocabulary->name = tt("taxonomy:vocabulary:$vocabulary->vid:name", $vocabulary->name);
    if (!empty($vocabulary->help)) {
      $vocabulary->help = tt("taxonomy:vocabulary:$vocabulary->vid:help", $vocabulary->help);
    }
    $terms = i18ntaxonomy_localize_terms($terms);
  }

  // Store collected data into relation structure.
  $relation->vocabulary = $vocabulary;
  $relation->terms = $terms;
  $relation->tabset = array();

  return TRUE;
}

/**
 * Translate strings for the given content type.
 */
function _subnodes_by_taxonomy_localize_content_type(&$type) {
  if (module_exists('i18ncontent')) {
    $type->name = tt("nodetype:type:$type->type:name", $type->name);
    if (!empty($type->description)) {
      $type->description = tt("nodetype:type:$type->type:description", $type->description);
    }
    if (!empty($type->title_label)) {
      $type->title_label = tt("nodetype:type:$type->type:title", $type->title_label);
    }
    if (!empty($type->body_label)) {
      $type->body_label = tt("nodetype:type:$type->type:body", $type->body_label);
    }
    if (!empty($type->help)) {
      $type->help = tt("nodetype:type:$type->type:help", $type->help);
    }
  }
}

/**
 * Sort callback; sort children tabs alphabetically.
 */
function _subnodes_by_taxonomy_sort_tabs($a, $b) {
  return ($a->name === $b->name ? 0 : strcmp(drupal_strtolower($a->name), drupal_strtolower($b->name)));
}
