<?php

/**
 * @file
 * Manage relations between content types in tabs.
 */

/**
 * Menu callback for node view tabs.
 */
function subnodes_by_taxonomy_node_view($mode, $parent_node, $child_node) {
  module_load_include('inc', 'subnodes_by_taxonomy', 'includes/common');

  if (($relation = subnodes_by_taxonomy_get_relation($parent_node->type, FALSE)) === FALSE) {
    // Invalid request.
    if ($mode == 'view') {
      drupal_not_found();
      return;
    }
    drupal_json(array('data' => ''));
    exit;
  }

  // When AJAX is not supported, we generate a page for the parent node
  // where the child node is rendered in its own tab.
  if ($mode == 'view') {
    // See theme_subnodes_by_taxonomy_tabset().
    $parent_node->_subnodes_by_taxonomy = array(
      'child_nid' => $child_node->nid,
    );
    return node_page_view($parent_node);
  }

  // See subnodes_by_taxonomy_nodeapi().
  $child_node->_subnodes_by_taxonomy = array('parent_node' => &$parent_node);

  // Render the child node and prepare the result.
  $output = node_view($child_node, FALSE, TRUE);
  $result = array('status' => TRUE, 'data' => theme('status_messages') ."\n". $output);
  drupal_alter('ajax_data', $result, 'subnodes_by_taxonomy', 'subnodes_by_taxonomy_node_view');

  // Using drupal_json() breaks filefield's file upload, because the jQuery
  // Form plugin handles file uploads in a way that is not compatible with
  // 'text/javascript' response type.
  $_GLOBALS['devel_shutdown'] =  FALSE;
  print drupal_to_js($result);
  exit;
}

/**
 * Menu callback for node edit form inside an iframe.
 */
function subnodes_by_taxonomy_node_edit_frame($mode, $parent_node, $child_node = NULL, $child_tid = NULL) {
  // Enable the Modal Frame API for the child window.
  modalframe_child_js();

  $module_path = drupal_get_path('module', 'subnodes_by_taxonomy');
  drupal_add_css($module_path .'/theme/subnodes_by_taxonomy.css');

  // Render the node edit form itself.
  module_load_include('inc', 'node', 'node.pages');

  if ($mode == 'edit') {
    // See subnodes_by_taxonomy_form_alter().
    $child_node->_subnodes_by_taxonomy = array(
      'mode' => $mode,
      'parent_node' => &$parent_node,
    );
    return node_page_edit($child_node);
  }
  elseif ($mode == 'create') {
    // See node_add() in node.pages.inc.
    global $user;
    $nodetype = str_replace('-', '_', $child_node);
    $type_name = node_get_types('name', $nodetype);
    if (module_exists('i18ncontent')) {
      $type_name = tt("nodetype:type:$nodetype:name", $type_name);
    }
    if ($type_name) {
      $child_node = (object)array(
        'uid' => $user->uid,
        'name' => (isset($user->name) ? $user->name : ''),
        'type' => $nodetype,
        'language' => '',
        // See subnodes_by_taxonomy_form_alter().
        '_subnodes_by_taxonomy' => array(
          'mode' => $mode,
          'parent_node' => &$parent_node,
          'child_tid' => $child_tid,
        ),
      );
      drupal_set_title(t('Create @name', array('@name' => $type_name)));
      return drupal_get_form($nodetype .'_node_form', $child_node);
    }
  }
  elseif ($mode == 'clone') {
    if (module_exists('clone')) {
      // See subnodes_by_taxonomy_form_alter().
      $child_node->_subnodes_by_taxonomy = array(
        'mode' => $mode,
        'parent_node' => &$parent_node,
        'child_tid' => $child_tid,
      );
      module_load_include('inc', 'clone', 'clone.pages');
      return clone_node_prepopulate($child_node);
    }
  }
  elseif ($mode == 'translate') {
    if (module_exists('translation')) {
      drupal_add_js(drupal_get_path('module', 'subnodes_by_taxonomy') .'/theme/child.translate.js');
      $js_settings = array(
        'parentNid' => $parent_node->nid,
      );
      drupal_add_js(array('subnodesByTaxonomy' => $js_settings), 'setting');

      if (module_exists('i18n')) {
        module_load_include('inc', 'i18n', 'i18n.pages');
        return i18n_translation_node_overview($child_node);
      }
      module_load_include('inc', 'translation', 'translation.pages');
      return translation_node_overview($child_node);
    }
  }
  elseif ($mode == 'relationships') {
    if (module_exists('noderelationships')) {
      module_load_include('inc', 'noderelationships', 'noderelationships.pages');
      return noderelationships_backref_page($child_node);
    }
  }
  elseif ($mode == 'delete') {
    return drupal_get_form('subnodes_by_taxonomy_child_delete_confirm', $parent_node, $child_node, $child_tid);
  }

  drupal_not_found();
}

/**
 * Menu callback -- ask for confirmation of child node deletion.
 */
function subnodes_by_taxonomy_child_delete_confirm(&$form_state, &$parent_node, &$child_node, $child_tid) {
  $form = array();
  $form['parent_nid'] = array('#type' => 'value', '#value' => $parent_node->nid);
  $form['child_nid'] = array('#type' => 'value', '#value' => $child_node->nid);
  $form['selector'] = array('#type' => 'value', '#value' => str_replace('_', '-', $child_node->type) .'-'. $child_node->nid);
  $form['child_tid'] = array('#type' => 'value', '#value' => $child_tid);

  drupal_set_title(t('Confirm delete operation'));
  $form['question'] = array('#value' => '<h2>'. t('Are you sure you want to delete %title?', array('%title' => $child_node->title)) .'</h2>');

  // Confirm form fails duplication check, as the form values rarely change -- so skip it.
  $form['#skip_duplicate_check'] = TRUE;

  $form['#attributes'] = array('class' => 'confirmation');
  $form['description'] = array('#value' => t('This action cannot be undone.'));
  $form['actions'] = array('#prefix' => '<div class="container-inline">', '#suffix' => '</div>');
  $form['actions']['submit'] = array('#type' => 'submit', '#value' => t('Delete'));
  return $form;
}

/**
 * Execute node deletion.
 */
function subnodes_by_taxonomy_child_delete_confirm_submit($form, &$form_state) {
  node_delete($form_state['values']['child_nid']);

  // Tell the parent window to close the modal frame and remove the tab
  // related to the child node being deleted.
  modalframe_close_dialog(array(
    'operation' => 'deleteTab',
    'selector' => $form_state['values']['selector'],
    'tid' => $form_state['values']['child_tid'],
  ));
}

/**
 * Submit handler for the node edit form of child nodes.
 *
 * @see subnodes_by_taxonomy_form_node_child_alter()
 */
function subnodes_by_taxonomy_form_node_child_submit($form, &$form_state) {
  $mode = $form['#subnodes_by_taxonomy']['mode'];
  if ($mode == 'edit') {
    modalframe_close_dialog(array(
      'operation' => 'updateTab',
      'selector' => $form_state['values']['selector'],
    ));
  }
  elseif ($mode == 'create' || $mode == 'clone') {
    $parent_node = &$form['#subnodes_by_taxonomy']['parent_node'];
    $relation = &$form['#subnodes_by_taxonomy']['relation'];
    $child_tid = $form['#subnodes_by_taxonomy']['child_tid'];

    if (($relation = subnodes_by_taxonomy_get_relation($parent_node->type, $parent_node->nid)) === FALSE) {
      drupal_set_message(t('Could not locate updated information about relations for this node.'), 'error');
    }
    else {
      $found = FALSE;
      foreach (array_values($relation->tabset) as $index => $child_tab) {
        if ($child_tab->tid == $child_tid) {
          if ($child_node = node_load($child_tab->nid)) {
            $found = $index;
          }
          break;
        }
      }
      if ($found === FALSE) {
        drupal_set_message(t('Could not locate information about the new child node.'), 'error');
      }
      else {
        // Tell the parent window to create a new tab for the new child.
        $subnode = theme('subnodes_by_taxonomy_tab_panel', $parent_node, $child_node, $child_tab);
        modalframe_close_dialog(array(
          'operation' => 'addTab',
          'index' => $index,
          'url' => url('node/'. $parent_node->nid .'/subnode/'. $child_tab->nid .'/view'),
          'label' => $child_tab->name,
          'title' => $relation->terms[$child_tid]->description,
          'selector' => str_replace('_', '-', $relation->child_type) .'-'. $child_tab->nid,
          'tid' => $child_tid,
          'panel' => $subnode['panel'],
        ));
      }
    }
  }
}

/**
 * Alter the node edit form to edit children in tabs by taxonomy.
 */
function subnodes_by_taxonomy_form_node_parent_alter(&$form, $form_state, &$relation) {
  $form['subnodes_by_taxonomy'] = array(
    '#theme' => 'subnodes_by_taxonomy_tabset',
    '#title' => $relation->vocabulary->name,
    '#subnodes-by-taxonomy' => array(
      'mode' => 'edit',
      'parent_node' => $form['#node'],
      'relation' => $relation,
    ),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#weight' => 1,
  );
}

/**
 * Alter the node edit form of children nodes.
 */
function subnodes_by_taxonomy_form_node_child_alter(&$form, $form_state, &$parent_node, &$relation, $child_tid) {
  $child_node = $form['#node'];
  $mode = $child_node->_subnodes_by_taxonomy['mode'];
  $child_nid = ($mode == 'edit' ? $child_node->nid : 0);

  // Pass the selector in edit mode. See subnodes_by_taxonomy_form_node_child_submit().
  if ($mode == 'edit') {
    $form['selector'] = array('#type' => 'value', '#value' => str_replace('_', '-', $child_node->type) .'-'. $child_nid);
  }

  // Assign relationship data to the new child node.
  if ($child_tid > 0) {
    $group_name = $relation->group_name;
    if (isset($form[$group_name])) {
      $element = &$form[$group_name];
    }
    else {
      $element = &$form;
    }
    $nodereference_field_name = $relation->fields['nodereference']['name'];
    if (isset($element[$nodereference_field_name])) {
      $element[$nodereference_field_name][0]['#default_value']['nid'] = $parent_node->nid;
    }
    $taxonomy_field_name = $relation->fields['content_taxonomy']['name'];
    if (isset($element[$taxonomy_field_name])) {
      $element[$taxonomy_field_name]['#default_value'][0]['value'] = $child_tid;
    }
  }

  // Attach a Pre-render handler so that we can alter form elements after
  // they are completely build.
  $form['#pre_render'][] = 'subnodes_by_taxonomy_child_node_pre_render';
  $form['#subnodes_by_taxonomy'] = array(
    'mode' => $mode,
    'relation' => &$relation,
    'parent_node' => &$parent_node,
    'child_tid' => $child_tid,
  );

  // Remove all buttons except the submit (Save).
  foreach (element_children($form['buttons']) as $key) {
    if ($key != 'submit' && isset($form['buttons'][$key]) && $form['buttons'][$key]) {
      unset($form['buttons'][$key]);
    }
  }

  // Enable the Modal Frame API for the child window.
  modalframe_child_js();

  $form['buttons']['submit']['#submit'][] = 'subnodes_by_taxonomy_form_node_child_submit';
}

/**
 * Display children in parent node.
 */
function subnodes_by_taxonomy_nodeapi_view_parent(&$parent_node, &$relation) {
  $parent_node->content['subnodes_by_taxonomy'] = array(
    '#theme' => 'subnodes_by_taxonomy_tabset',
    '#title' => $relation->vocabulary->name,
    '#subnodes-by-taxonomy' => array(
      'mode' => 'view',
      'parent_node' => $parent_node,
      'relation' => $relation,
    ),
    '#collapsible' => FALSE,
    '#weight' => subnodes_by_taxonomy_get_element_weight($parent_node->type),
  );
}

/**
 * Alter display of child nodes.
 */
function subnodes_by_taxonomy_nodeapi_view_child(&$parent_node, &$child_node, &$relation) {
  // Attach a Pre-render handler so that we can alter form elements after
  // they are completely build.
  $child_node->content['#pre_render'][] = 'subnodes_by_taxonomy_child_node_pre_render';
  $child_node->content['#subnodes_by_taxonomy'] = array(
    'mode' => 'view',
    'relation' => &$relation,
    'parent_node' => &$parent_node,
  );
}

/**
 * Pre-render callback for the node edit form.
 */
function subnodes_by_taxonomy_child_node_pre_render($elements) {
  if (isset($elements['#subnodes_by_taxonomy']) && !empty($elements['#subnodes_by_taxonomy']['relation'])) {
    $mode = $elements['#subnodes_by_taxonomy']['mode'];
    $relation = &$elements['#subnodes_by_taxonomy']['relation'];
    $parent_node = &$elements['#subnodes_by_taxonomy']['parent_node'];
    $child_node = &$elements['#node'];
    $child_tid = (isset($elements['#subnodes_by_taxonomy']['child_tid']) ? $elements['#subnodes_by_taxonomy']['child_tid'] : 0);
    $group_name = $relation->group_name;

    if (isset($child_node->nid)) {
      $child_tab = (isset($relation->tabset[$child_node->nid]) ? $relation->tabset[$child_node->nid] : NULL);
    }
    else {
      $child_tab = (isset($relation->tabset[0]) ? $relation->tabset[0] : NULL);
    }

    // Disable the node title.
    if (isset($elements['title']) && isset($child_tab)) {
      $elements['title']['#value'] = trim($parent_node->title) .' ('. trim($relation->terms[$child_tab->tid]->name) .')';
      $elements['title'] = array(
        'title' => $elements['title'],
        '_disabled_title' => $elements['title'],
      );
      $elements['title']['title']['#type'] = 'hidden';
      _subnodes_by_taxonomy_disable_element($elements['title']['_disabled_title']);
    }

    // Hide the group that contains the parent/child relations.
    if (isset($elements[$group_name])) {
      $element = &$elements[$group_name];
      if ($mode == 'view' && isset($element['group'])) {
        $element = &$element['group'];
      }
      _subnodes_by_taxonomy_hide_element($element);
    }

    // Unset our data so that we don't execute our Pre-render code twice.
    // Note that the node form is rendered once by FAPI, and twice by
    // theme_node_form() in node.pages.inc.
    unset($elements['#subnodes_by_taxonomy']);
  }
  return $elements;
}

/**
 * Helper function to hide an element.
 */
function _subnodes_by_taxonomy_hide_element(&$element) {
  if (!isset($element['#attributes'])) {
    $element['#attributes'] = array();
  }
  if (empty($element['#attributes']['class'])) {
    $element['#attributes']['class'] = 'subnodes-by-taxonomy-hidden';
  }
  else {
    $element['#attributes']['class'] .= ' subnodes-by-taxonomy-hidden';
  }
}

/**
 * Recursively set the disabled attribute of an element and children.
 */
function _subnodes_by_taxonomy_disable_element(&$elements) {
  foreach (element_children($elements) as $key) {
    if (isset($elements[$key]) && $elements[$key]) {
      // Recurse through all children elements.
      _subnodes_by_taxonomy_disable_element($elements[$key]);
    }
  }
  if (!isset($elements['#attributes'])) {
    $elements['#attributes'] = array();
  }
  $elements['#attributes']['disabled'] = 'disabled';
}

/**
 * Render a tabset for the parent node.
 */
function theme_subnodes_by_taxonomy_tabset($element) {
  $mode = $element['#subnodes-by-taxonomy']['mode'];
  $parent_node = $element['#subnodes-by-taxonomy']['parent_node'];
  $relation = $element['#subnodes-by-taxonomy']['relation'];

  if (isset($parent_node->_subnodes_by_taxonomy) && !empty($parent_node->_subnodes_by_taxonomy['child_nid'])) {
    $selected_child_nid = (int)$parent_node->_subnodes_by_taxonomy['child_nid'];
  }
  else {
    $selected_child_nid = 0;
  }

  $module_path = drupal_get_path('module', 'subnodes_by_taxonomy');
  drupal_add_css($module_path .'/theme/subnodes_by_taxonomy.css');
  drupal_add_js($module_path .'/theme/parent.js');

  // Enable the Modal Frame API for the parent window.
  modalframe_parent_js();

  // Prepare the list of terms for children tabs.
  $children_terms = array();
  foreach ($relation->terms as $tid => $term) {
    $children_terms[$tid] = array(
      'name' => check_plain($term->name),
      'title' => check_plain(!empty($term->description) ? $term->description : $term->name),
      'available' => TRUE,
    );
  }

  $tabs = array();
  $panels = array();
  foreach ($relation->tabset as $child_tab) {
    if (($child_node = node_load($child_tab->nid)) && subnodes_by_taxonomy_node_access('view', $parent_node, $child_node)) {
      if (isset($children_terms[$child_tab->tid])) {
        $children_terms[$child_tab->tid]['available'] = FALSE;
      }
      $subnode = theme('subnodes_by_taxonomy_tab_panel', $parent_node, $child_node, $child_tab, $selected_child_nid);
      $tabs[] = $subnode['tab'];
      $panels[] = $subnode['panel'];
    }
  }

  // Send the list of terms for children tabs.
  $js_settings = array(
    'childrenTermsTitle' => check_plain(t($relation->fields['content_taxonomy']['label'])),
    'childrenTerms' => $children_terms,
  );
  drupal_add_js(array('subnodesByTaxonomy' => $js_settings), 'setting');

  // Build the 'Add new...' tab.
  $nodetype = $relation->child_type;
  if (isset($parent_node->nid) && subnodes_by_taxonomy_node_access('create', $parent_node, $nodetype)) {
    $type_name = node_get_types('name', $nodetype);
    if (module_exists('i18ncontent')) {
      $type_name = tt("nodetype:type:$nodetype:name", $type_name);
    }
    $url_path = 'node/'. $parent_node->nid .'/subnode/add/'. $nodetype;
    $url_options = array('attributes' => array(
      'title' => t('Add new !type-name', array('!type-name' => $type_name)),
      'rel' => str_replace('_', '-', $nodetype) .'-add',
    ));
    $tabs[] = '<li class="subnodes-by-taxonomy-tabset-addtab">'. l(t('Add new...'), $url_path, $url_options) .'</li>';
  }

  $output = '';
  if (!empty($tabs)) {
    $nav_attributes = 'id="subnodes-by-taxonomy-tabset-'. $parent_node->nid .'" class="subnodes-by-taxonomy-tabset-nav"';
    $output .= '<div class="subnodes-by-taxonomy-tabset-wrapper">';
    $output .= '<div '. $nav_attributes .'><ul class="clear-block">'."\n". implode("\n", $tabs) ."\n".'</ul></div>'."\n";
    $output .= implode("\n", $panels);
    $output .= '</div>';
  }
  else {
    if ($mode == 'edit' && !isset($parent_node->nid) && subnodes_by_taxonomy_node_access('create', $parent_node, $relation->child_type)) {
      $element['#collapsed'] = TRUE;
      $warning = t('You will not be able to create %child-vocabulary until this %parent-type is saved into the database.', array(
        '%parent-type' => $relation->types['parent']->name,
        '%child-vocabulary' => $relation->vocabulary->name,
      ));
      $output = '<div class="warning">'. $warning .'<div>';
    }
  }

  if (!empty($output)) {
    $content = array();
    $content['subnodes_by_taxonomy'] = array(
      '#type' => 'fieldset',
      '#title' => $element['#title'],
      '#collapsible' => $element['#collapsible'],
      '#collapsed' => $element['#collapsed'],
    );
    $content['subnodes_by_taxonomy']['tabs'] = array(
      '#type' => 'markup',
      '#value' => $output,
    );
    $output = drupal_render($content);
  }

  return $output;
}

/**
 * Render subnode tab and panel.
 */
function theme_subnodes_by_taxonomy_tab_panel(&$parent_node, &$child_node, &$child_tab, $selected_child_nid = 0) {
  $tab_title = $child_tab->description;
  $panel_id = str_replace('_', '-', $child_node->type) .'-'. $child_tab->nid;
  $url_path = 'node/'. $parent_node->nid .'/subnode/'. $child_tab->nid .'/view';
  $url_options = array('attributes' => array('title' => $tab_title, 'rel' => $panel_id));
  $panel_class = 'subnodes-by-taxonomy-panel';
  $tasks = '';
  if ($selected_child_nid > 0 && $selected_child_nid == $child_tab->nid) {
    $child_node->_subnodes_by_taxonomy = array('parent_node' => &$parent_node);
    $child_view = node_view($child_node, FALSE, TRUE);
    $child_class = ' class="subnodes-by-taxonomy-selected"';
  }
  else {
    $child_view = $child_class = '';
    $panel_class .= ' subnodes-by-taxonomy-hidden';
    // Generate task links only when no child specified in URL.
    if (!$selected_child_nid) {
      $tasks = theme('subnodes_by_taxonomy_tab_tasks', $parent_node, $child_node, $panel_id, $child_tab->tid);
    }
  }
  $title = '<h2><span>'. check_plain($parent_node->title) .'</span> ('. check_plain($tab_title) .')</h2>';
  $panel = '<div class="subnodes-by-taxonomy-child-caption clear-block">'. $title . $tasks .'</div>';
  $panel .= '<div class="subnodes-by-taxonomy-child-body">'. $child_view .'</div>';
  return array(
    'tab' => '<li'. $child_class .'>'. l($child_tab->name, $url_path, $url_options) .'</li>',
    'panel' => '<div id="'. $panel_id .'" class="'. $panel_class .'"><div class="clear-block">'. $panel .'</div></div>',
  );
}

/**
 * Render subnode tasks.
 */
function theme_subnodes_by_taxonomy_tab_tasks(&$parent_node, &$child_node, $panel_id, $child_tid) {
  $items = array();
  if (subnodes_by_taxonomy_node_access('update', $parent_node, $child_node)) {
    $items[] = l(t('Edit'), 'node/'. $parent_node->nid .'/subnode/'. $child_node->nid .'/edit', array('attributes' => array(
      'class' => 'subnodes-by-taxonomy-dialog-task',
      'title' => t('Edit !title', array('!title' => $child_node->title)),
      'rel' => $panel_id,
    )));
  }
  if (subnodes_by_taxonomy_node_access('clone', $parent_node, $child_node)) {
    $items[] = l(t('Clone'), 'node/'. $parent_node->nid .'/subnode/'. $child_node->nid .'/clone', array('attributes' => array(
      'class' => 'subnodes-by-taxonomy-dialog-task subnodes-by-taxonomy-dialog-task-addtab',
      'title' => t('Clone !title', array('!title' => $child_node->title)),
      'rel' => $panel_id,
    )));
  }
  if (subnodes_by_taxonomy_node_access('translate', $parent_node, $child_node)) {
    $items[] = l(t('Translate'), 'node/'. $parent_node->nid .'/subnode/'. $child_node->nid .'/translate', array('attributes' => array(
      'class' => 'subnodes-by-taxonomy-dialog-task',
      'title' => t('Translate !title', array('!title' => $child_node->title)),
      'rel' => $panel_id,
    )));
  }
  if (subnodes_by_taxonomy_node_access('relationships', $parent_node, $child_node)) {
    $items[] = l(t('Relationships'), 'node/'. $parent_node->nid .'/subnode/'. $child_node->nid .'/relationships', array('attributes' => array(
      'class' => 'subnodes-by-taxonomy-dialog-task',
      'title' => t('Relationships for !title', array('!title' => $child_node->title)),
      'rel' => $panel_id,
    )));
  }
  if (subnodes_by_taxonomy_node_access('delete', $parent_node, $child_node)) {
    $items[] = l(t('Delete'), 'node/'. $parent_node->nid .'/subnode/'. $child_node->nid .'/delete/'. $child_tid, array('attributes' => array(
      'class' => 'subnodes-by-taxonomy-dialog-task subnodes-by-taxonomy-dialog-size[400,200]',
      'title' => t('Delete !title', array('!title' => $child_node->title)),
      'rel' => $panel_id,
    )));
  }
  $output = '';
  if (!empty($items)) {
    $output .= '<ul class="subnodes-by-taxonomy-subnode-tasks">';
    $output .= implode('', array_map(create_function('$a', 'return "<li>". $a ."</li>";'), $items));
    $output .= '</ul>';
  }
  return $output;
}
